# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""

PyTrilinos.TriUtils is the python interface to the Trilinos utilities
package TriUtils:

    https://trilinos.org/docs/dev/packages/triutils/doc/html/index.html

The purpose of TriUtils is to provide some utilities typically needed
when testing Trilinos software.

TriUtils provides the following user-level classes:

    * CrsMatrixGallery  - Provide example CrsMatrix objects
    * VbrMatrixGallery  - Provide example VbrMatrix objects

and function:

    * ReadHB            - Obtain a problem from an HB file

For an examples of usage, please consult the following script in the
example subdirectory of the PyTrilinos package:

    * exIFPACK.py

"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _TriUtils
else:
    import _TriUtils

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

from . import Epetra

def TriUtils_Version(*args) -> "std::string":
    return _TriUtils.TriUtils_Version(*args)

__version__ = TriUtils_Version().split()[3]

MAXNRHS = _TriUtils.MAXNRHS
class CrsMatrixGallery(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        *Overload 1:*
        Triutils_Gallery Constructor. Creates a Triutils_Gallery instance.

         The first parameter is the name of the matrix. We refer to the Trilinos
         Tutorial for a detailed description of available matrices.

         Notes: The matrix name can be empty (""), and set later using, for example,
         Set("matrix_name","laplace_2d");

         An example of program using this class is reported below.

         .. code-block:: c++

           int main(int argc, char *argv[])
           {

           #ifdef HAVE_MPI
             MPI_Init(&argc,&argv);
             Epetra_MpiComm Comm (MPI_COMM_WORLD);
           #else
             Epetra_SerialComm Comm;
           #endif

           create an Epetra matrix reading an H/B matrix
             Trilinos_Util_CrsMatrixGallery Gallery("hb", Comm);

           set the name of the matrix
             Gallery.Set("matrix name", "bcsstk14.rsa");

             Epetra_CrsMatrix * A;
             Epetra_Vector * ExactSolution;
             Epetra_Vector * RHS;
             Epetra_Vector * StartingSolution;

           at this point the matrix is read from file
             A = Gallery.GetMatrix();
             ExactSolution = Gallery.GetExactSolution();

           at this point the RHS is allocated and filled
             RHS = Gallery.GetRHS();
             StartingSolution = Gallery.GetStartingSolution();

           create linear problem
             Epetra_LinearProblem Problem(A,StartingSolution,RHS);
           create AztecOO instance
             AztecOO Solver(Problem);

             Solver.SetAztecOption( AZ_precond, AZ_dom_decomp );
             Solver.Iterate(1000,1E-9);

           compute residual
             double residual;

             Gallery.ComputeResidual(&residual);
             if( Comm.MyPID()==0 ) cout << "||b-Ax||_2 = " << residual << endl;

             Gallery.ComputeDiffBetweenStartingAndExactSolutions(&residual);
             if( Comm.MyPID()==0 ) cout << "||x_exact - x||_2 = " << residual << endl;

            #ifdef HAVE_MPI
             MPI_Finalize() ;
           #endif

           return 0 ;
             }

         Class CommandLineParser can be used as well. In this case, one may
         decide to use the following:

         .. code-block:: c++

             Trilinos_Util::CommandLineParser CLP(argc,argv);
           set a problem with no matrix name
             Trilinos_Util::CrsMatrixGallery Gallery("", Comm);
           read parameters and settings from the shell line
             G.Set(CLP);
           continue with your code...

         :param In:
             comm - Epetra communicator

        |

        *Overload 2:*
        Creates an Triutils_Gallery object using a given map. Create a Triutils_Gallery object using an Epetra_Map.
           Problem size must match the elements in map.

           :param In:
               name - definition of the problem to be created.

           :param In:
               map - Epetra_Map
        """
        _TriUtils.CrsMatrixGallery_swiginit(self, _TriUtils.new_CrsMatrixGallery(*args))
    __swig_destroy__ = _TriUtils.delete_CrsMatrixGallery

    def Set(self, *args) -> "int":
        r"""
        *Overload 1:*
        Sets a gallery options using an interger value.

        |

        *Overload 2:*
         Sets a gallery options using a C++ string .

        |

        *Overload 3:*
        Sets a gallery options using an double value.

        |

        *Overload 4:*
        Sets a gallery options using an Epetra_Vector. Sets a gallery options using an Epetra_Vector. The Epetra_Vector
         is copied into internal structures, and freed by the destructor.

        |

        *Overload 5:*
        Sets gallery options using values passed from the shell
        """
        return _TriUtils.CrsMatrixGallery_Set(self, *args)

    def GetMatrix(self, *args) -> "Epetra_CrsMatrix *":
        r""" Returns a pointer to the CrsMatrix."""
        return _TriUtils.CrsMatrixGallery_GetMatrix(self, *args)

    def GetMatrixRef(self, *args) -> "Epetra_CrsMatrix &":
        return _TriUtils.CrsMatrixGallery_GetMatrixRef(self, *args)

    def GetExactSolution(self, *args) -> "Epetra_MultiVector *":
        r"""
        Returns a pointer to the exact solution. Returns a pointer to the exact solution.

           Some choices are available to define the exact solution, using
           Set("exact solution", value). value can be:
           - constant: the exact solution vector is made up of 1's.
           - random: a random solution vector
           - linear: value at node i is defined as alpha*i. The double value
           alpha can be set via Set("alpha",DoubleVal).
        """
        return _TriUtils.CrsMatrixGallery_GetExactSolution(self, *args)

    def GetStartingSolution(self, *args) -> "Epetra_MultiVector *":
        r"""
        Returns a pointer to the starting solution (typically, for HB problems). Returns a pointer to the starting solution. This is typically used
           while reading a HB problem. However, the user can set a starting
           solution using Set("starting solution", "value"). Value can be
           - zero
           - random
        """
        return _TriUtils.CrsMatrixGallery_GetStartingSolution(self, *args)

    def GetRHS(self, *args) -> "Epetra_MultiVector *":
        r""" Returns a pointer to the rhs corresponding to the selected exact solution."""
        return _TriUtils.CrsMatrixGallery_GetRHS(self, *args)

    def GetMap(self, *args) -> "Epetra_Map const *":
        r""" Returns a pointer the internally stored Map."""
        return _TriUtils.CrsMatrixGallery_GetMap(self, *args)

    def GetMapRef(self, *args) -> "Epetra_Map const &":
        return _TriUtils.CrsMatrixGallery_GetMapRef(self, *args)

    def GetLinearProblem(self, *args) -> "Epetra_LinearProblem *":
        r""" Returns a pointer to Epetra_LinearProblem"""
        return _TriUtils.CrsMatrixGallery_GetLinearProblem(self, *args)

    def ComputeResidual(self, *args) -> "void":
        r""" Computes the 2-norm of the residual"""
        return _TriUtils.CrsMatrixGallery_ComputeResidual(self, *args)

    def ComputeDiffBetweenStartingAndExactSolutions(self, *args) -> "void":
        r""" Computes the 2-norm of the difference between the starting solution and the exact solution"""
        return _TriUtils.CrsMatrixGallery_ComputeDiffBetweenStartingAndExactSolutions(self, *args)

    def PrintMatrixAndVectors(self, *args) -> "void":
        return _TriUtils.CrsMatrixGallery_PrintMatrixAndVectors(self, *args)

    def GetCartesianCoordinates(self, *args) -> "void":
        r""" Get pointers to double vectors containing coordinates of points."""
        return _TriUtils.CrsMatrixGallery_GetCartesianCoordinates(self, *args)

    def WriteMatrix(self, *args) -> "int":
        r""" Print matrix on file in MATLAB format"""
        return _TriUtils.CrsMatrixGallery_WriteMatrix(self, *args)

# Register CrsMatrixGallery in _TriUtils:
_TriUtils.CrsMatrixGallery_swigregister(CrsMatrixGallery)

class VbrMatrixGallery(CrsMatrixGallery):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _TriUtils.VbrMatrixGallery_swiginit(self, _TriUtils.new_VbrMatrixGallery(*args))
    __swig_destroy__ = _TriUtils.delete_VbrMatrixGallery

    def GetBlockMap(self, *args) -> "Epetra_BlockMap const *":
        r""" Returns a pointer the internally stored BlockMap."""
        return _TriUtils.VbrMatrixGallery_GetBlockMap(self, *args)

    def GetBlockMapRef(self, *args) -> "Epetra_BlockMap const &":
        return _TriUtils.VbrMatrixGallery_GetBlockMapRef(self, *args)

    def GetVbrMatrix(self, *args) -> "Epetra_VbrMatrix *":
        r"""
        *Overload 1:*
        Returns a VbrMatrix, starting from the CsrMatrix. Returns a VbrMatrix, starting from the CsrMatrix. This vbr matrix
           is formally equivalent to the CrsMatrix returned by
           GetMatrix(). However, each node of the CrsMatrix is replicated
           num_PDE_eqns times (this value is passed in input, or set via Set("num pde
           eqns",IntValue)).

        |

        *Overload 2:*
        Returns a VbrMatrix, starting from the CsrMatrix.
        """
        return _TriUtils.VbrMatrixGallery_GetVbrMatrix(self, *args)

    def GetVbrMatrixRef(self, *args) -> "Epetra_VbrMatrix &":
        return _TriUtils.VbrMatrixGallery_GetVbrMatrixRef(self, *args)

    def GetVbrRHS(self, *args) -> "Epetra_MultiVector *":
        r""" Returns a pointer to the RHS for the selected Vbr exact solution  Returns a pointer to the RHS  corresponding to the selected exact solution to the linear systems defined by the Epetra_VbrMatrix."""
        return _TriUtils.VbrMatrixGallery_GetVbrRHS(self, *args)

    def GetVbrExactSolution(self, *args) -> "Epetra_MultiVector *":
        r""" Returns a pointer to the selected Vbr exact solution"""
        return _TriUtils.VbrMatrixGallery_GetVbrExactSolution(self, *args)

    def GetVbrStartingSolution(self, *args) -> "Epetra_MultiVector *":
        r""" Returns a pointer to the starting solution for Vbr problems"""
        return _TriUtils.VbrMatrixGallery_GetVbrStartingSolution(self, *args)

    def CreateVbrMatrix(self, *args) -> "void":
        return _TriUtils.VbrMatrixGallery_CreateVbrMatrix(self, *args)

    def GetVbrLinearProblem(self, *args) -> "Epetra_LinearProblem *":
        r""" Returns a pointer to Epetra_LinearProblem for VBR"""
        return _TriUtils.VbrMatrixGallery_GetVbrLinearProblem(self, *args)

    def ComputeResidualVbr(self, *args) -> "void":
        r""" Computes the 2-norm of the residual for the VBR problem"""
        return _TriUtils.VbrMatrixGallery_ComputeResidualVbr(self, *args)

    def ComputeDiffBetweenStartingAndExactSolutionsVbr(self, *args) -> "void":
        r""" Computes the 2-norm of the difference between the starting solution and the exact solution for the VBR problem"""
        return _TriUtils.VbrMatrixGallery_ComputeDiffBetweenStartingAndExactSolutionsVbr(self, *args)

    def PrintVbrMatrixAndVectors(self, *args) -> "void":
        return _TriUtils.VbrMatrixGallery_PrintVbrMatrixAndVectors(self, *args)

# Register VbrMatrixGallery in _TriUtils:
_TriUtils.VbrMatrixGallery_swigregister(VbrMatrixGallery)



